/*
 *$Header: /REMET SDK/general/rmtOSVersion.h 11    17.09.07 17:37 Kyselgov $ 
*/

#ifndef _RMTOSVERSION_H__INCLUDED_
#define _RMTOSVERSION_H__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "crtdbg.h"

namespace rmt
{
	namespace Platform
	{
		/*!
		\defgroup group_rmtPlatform_OS  

		   , , , -  .. 
		     ,   .

		\ingroup group_rmtPlatform
		*/

		namespace OS
		{
			/** \addtogroup group_rmtPlatform_OS */
			/*@{*/

			__declspec(selectany) OSVERSIONINFOEX gOS = {0};

			/*! 
			\brief   .

			       ,  
			  .       
			     ,       
			     gOS,      
			         COSVersion.

			*/
			class __declspec(novtable) COSVersion
			{
				// 
			public:

				COSVersion()
				{
					FillData();
				}

				inline const OSVERSIONINFOEX& GetVersionInfo() const;	
				inline bool		IsWin98() const;
				inline bool		IsWin95() const;
				inline bool		IsWinMe() const;
				inline bool		IsWin2000() const;
				inline bool		IsWinXP() const;
				inline bool		IsWinXPSP1() const;
				inline bool		IsWin2003() const;
				inline bool		IsWinVista() const;
				inline bool		IsWin98orHigher() const;
				inline bool		IsWin95orHigher() const;
				inline bool		IsWinMeorHigher() const;
				inline bool		IsWin2000orHigher() const;
				inline bool		IsWinXPorHigher() const;
				inline bool		IsWinXPSP1orHigher() const;
				inline bool		IsWinXPSP2orHigher() const;
				inline bool		IsWin2003orHigher() const;
				inline bool		IsWinVistaorHigher() const;

				inline bool		IsNTPlatform() const;

			private:
				inline void		FillData();

			private:
				//        
				COSVersion( const COSVersion& ) throw();
				COSVersion& operator=( const COSVersion& ) throw();

			};



			/*!
			\brief  Windows 98 ?

			 -       COSVersion 
			 true   Windows 98.

			*/
			inline bool			infoIsWin98();

			/*!
			\brief  Windows 98     ?

			 -       COSVersion 
			 true   Windows 98    , : Windows ME, 2000, XP  ..

			*/
			inline bool			infoIsWin98orHigher();

			/*!
			\brief  Windows 2000     ?

			 -       COSVersion 
			 true   Windows 2000    , : Windows XP, 2003  ..

			*/
			inline bool			infoIsWin2000orHigher();

			/*!
			\brief  Windows XP     ?

			 -       COSVersion 
			 true   Windows XP    , : Windows 2003  ..

			*/
			inline bool			infoIsWinXPorHigher();

			/*!
			\brief  Windows XP SP 2     ?

			 -       COSVersion 
			 true   Windows XP Service Pack 2    , : Windows 2003  ..

			*/
			inline bool			infoIsWinXPSP2orHigher();

			/*!
			\brief  Windows Vista     ?

			 -       COSVersion 
			 true   Windows Vista    .

			*/
			inline bool			infoIsWinVistaorHigher();
			/*@}*/
		}
	}
}










///////////////////////////////////////////////////////////////////////////////
//  Inline 
//-----------------------------------------------------------------------------

namespace rmt
{
	namespace Platform
	{
		namespace OS
		{

			void COSVersion::FillData()
			{
				if (gOS.dwOSVersionInfoSize==0)
				{
					gOS.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);

					//     ...
					if (::GetVersionEx(reinterpret_cast<LPOSVERSIONINFO>(&gOS))!=FALSE)
					{
						//   ...
					}
					else
					{
						// ,    ...

						ZeroMemory(&gOS,sizeof(gOS));
						gOS.dwOSVersionInfoSize	= sizeof(OSVERSIONINFO);

						if (::GetVersionEx(reinterpret_cast<LPOSVERSIONINFO>(&gOS))!=FALSE)
						{
							//   ...
						}
						else
						{
							//    ...   ...
							_ASSERT(FALSE);
						}
					}
				}
			}

			const OSVERSIONINFOEX& COSVersion::GetVersionInfo() const
			{
				return gOS;
			}

			bool COSVersion::IsWin98() const
			{
				const bool bRes = (gOS.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS) & ((gOS.dwMajorVersion==4) & (gOS.dwMinorVersion==10));
				return bRes;
			}

			bool COSVersion::IsWin95() const
			{
				const bool bRes = (gOS.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS) & ((gOS.dwMajorVersion==4) & (gOS.dwMinorVersion==0));
				return bRes;
			}

			bool COSVersion::IsWinMe() const
			{
				const bool bRes = (gOS.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS) & ((gOS.dwMajorVersion==4) & (gOS.dwMinorVersion==90));
				return bRes;
			}

			bool COSVersion::IsWin2000() const
			{
				const bool bRes = (gOS.dwPlatformId == VER_PLATFORM_WIN32_NT) & ((gOS.dwMajorVersion>=5) & (gOS.dwMinorVersion==0));
				return bRes;
			}

			bool COSVersion::IsWinXP() const
			{
				const bool bRes = (gOS.dwPlatformId == VER_PLATFORM_WIN32_NT) & ((gOS.dwMajorVersion>=5) & (gOS.dwMinorVersion==1));
				return bRes;
			}

			bool COSVersion::IsWinXPSP1() const
			{
				const bool bRes = IsWinXP() & ((gOS.wServicePackMajor==1) & (gOS.wServicePackMinor==0));
				return bRes;
			}

			bool COSVersion::IsWin2003() const
			{
				const bool bRes = (gOS.dwPlatformId == VER_PLATFORM_WIN32_NT) & ((gOS.dwMajorVersion>=5) & (gOS.dwMinorVersion==2));
				return bRes;
			}

			bool COSVersion::IsWinVista() const
			{
				const bool bRes = (gOS.dwPlatformId == VER_PLATFORM_WIN32_NT) & ((gOS.dwMajorVersion==6) & (gOS.dwMinorVersion==0));
				return bRes;
			}

			bool COSVersion::IsWin98orHigher() const
			{
				const bool bRes = (gOS.dwPlatformId != VER_PLATFORM_WIN32s) & ( (gOS.dwMajorVersion>4) | ((gOS.dwMajorVersion==4) & (gOS.dwMinorVersion>=10)) );
				return bRes;
			}

			bool COSVersion::IsWin95orHigher() const
			{
				const bool bRes = (gOS.dwPlatformId != VER_PLATFORM_WIN32s) & (gOS.dwMajorVersion>=4);
				return bRes;
			}

			bool COSVersion::IsWinMeorHigher() const
			{
				const bool bRes = (gOS.dwPlatformId != VER_PLATFORM_WIN32s) & ( (gOS.dwMajorVersion>4) | ((gOS.dwMajorVersion==4) & (gOS.dwMinorVersion>=90)) );
				return bRes;
			}

			bool COSVersion::IsWin2000orHigher() const
			{
				const bool bRes = (gOS.dwPlatformId == VER_PLATFORM_WIN32_NT) & (gOS.dwMajorVersion>=5);
				return bRes;
			}

			bool COSVersion::IsWinXPorHigher() const
			{
				const bool bRes = (gOS.dwPlatformId == VER_PLATFORM_WIN32_NT) & ((gOS.dwMajorVersion>5) | ((gOS.dwMajorVersion==5) & (gOS.dwMinorVersion>=1)));
				return bRes;
			}

			bool COSVersion::IsWinXPSP1orHigher() const
			{
				const bool bRes = (gOS.dwPlatformId == VER_PLATFORM_WIN32_NT) & ((gOS.dwMajorVersion>5) | ((gOS.dwMajorVersion==5) & ((gOS.dwMinorVersion>1) | ((gOS.dwMinorVersion==1) & (gOS.wServicePackMajor>=1)))));
				return bRes;
			}

			bool COSVersion::IsWinXPSP2orHigher() const
			{
				const bool bRes = (gOS.dwPlatformId == VER_PLATFORM_WIN32_NT) & ((gOS.dwMajorVersion>5) | ((gOS.dwMajorVersion==5) & ((gOS.dwMinorVersion>1) | ((gOS.dwMinorVersion==1) & (gOS.wServicePackMajor>=2)))));
				return bRes;
			}

			bool COSVersion::IsWin2003orHigher() const
			{
				const bool bRes = (gOS.dwPlatformId == VER_PLATFORM_WIN32_NT) & ((gOS.dwMajorVersion>5) | ((gOS.dwMajorVersion==5) & (gOS.dwMinorVersion>=2)));
				return bRes;
			}

			bool COSVersion::IsWinVistaorHigher() const
			{
				const bool bRes = (gOS.dwPlatformId == VER_PLATFORM_WIN32_NT) & (gOS.dwMajorVersion>=6);
				return bRes;
			}

			bool COSVersion::IsNTPlatform() const
			{
				const bool bRes = (gOS.dwPlatformId == VER_PLATFORM_WIN32_NT);
				return bRes;
			}

			bool infoIsWin98()
			{
				//   
				COSVersion gOS;
				return gOS.IsWin98();
			}

			bool infoIsWin98orHigher()
			{
				//   
				COSVersion gOS;
				return gOS.IsWin98orHigher();
			}

			bool infoIsWin2000orHigher()
			{
				//   
				COSVersion gOS;
				return gOS.IsWin2000orHigher();
			}

			bool infoIsWinXPorHigher()
			{
				//   
				COSVersion gOS;
				return gOS.IsWinXPorHigher();
			}

			bool infoIsWinXPSP2orHigher()
			{
				//   
				COSVersion gOS;
				return gOS.IsWinXPSP2orHigher();
			}

			bool infoIsWinVistaorHigher()
			{
				//   
				COSVersion gOS;
				return gOS.IsWinVistaorHigher();
			}
			

		}
	}
}

//-----------------------------------------------------------------------------
//  Inline 
///////////////////////////////////////////////////////////////////////////////

#endif